/**
 *  Copyright (c) 2008-2011 Sonatype, Inc. All rights reserved.
 *
 *  This program is licensed to you under the Apache License Version 2.0,
 *  and you may not use this file except in compliance with the Apache License Version 2.0.
 *  You may obtain a copy of the Apache License Version 2.0 at http://www.apache.org/licenses/LICENSE-2.0.
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the Apache License Version 2.0 is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the Apache License Version 2.0 for the specific language governing permissions and limitations there under.
 */
package org.codehaus.plexus.swizzle;

import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.inject.Named;

import org.sonatype.sisu.pr.bundle.Bundle;

@Named
public class IssueSubmissionRequest
{
    private String issueTypeId;

    private String priorityId;

    private String affectsVersionId;

    private String fixVersionId;

    private String summary;

    private String description;

    private String reporter;

    private String assignee;

    private final List<File> problemReportBundles = new ArrayList<File>();

    private final List<File> screenCaptures = new ArrayList<File>();

    private String environment;

    private String component;

    private boolean validateRequest = true;

    private String projectKey;

    private Object context;

    // private static Logger logger = LoggerFactory.getLogger( IssueSubmissionRequest.class );

    private Collection<Bundle> bundles;

    private Throwable error;

    private boolean autofill = true;

    private boolean skipSubmission;

    public IssueSubmissionRequest()
    {
    }

    public IssueSubmissionRequest( final Throwable error )
    {
        setError( error );
    }

    /**
     * As the client if you have validated all the requirement already, skip checking against the server and the
     * validation of the request to speed up submission of issues to the server.
     *
     * @return
     */
    public boolean isValidateRequest()
    {
        return validateRequest;
    }

    public void setValidateRequest( final boolean validateRequest )
    {
        this.validateRequest = validateRequest;
    }

    public void setIssueTypeId( final String issueTypeId )
    {
        this.issueTypeId = issueTypeId;
    }

    public String getIssueTypeId()
    {
        return issueTypeId;
    }

    public String getPriorityId()
    {
        return priorityId;
    }

    public void setPriorityId( final String priorityId )
    {
        this.priorityId = priorityId;
    }

    public String getAffectsVersionId()
    {
        return affectsVersionId;
    }

    public void setAffectsVersionId( final String affectsVersionId )
    {
        this.affectsVersionId = affectsVersionId;
    }

    public String getFixVersionId()
    {
        return fixVersionId;
    }

    public void setFixVersionId( final String fixVersionId )
    {
        this.fixVersionId = fixVersionId;
    }

    /**
     * This method actually uses the project key, not id.
     */
    @Deprecated
    public String getProjectId()
    {
        return projectKey;
    }

    /**
     * This method actually uses the project key, not id.
     */
    @Deprecated
    public void setProjectId( final String projectId )
    {
        this.projectKey = projectId;
    }

    public void setProjectKey( final String projectKey )
    {
        this.projectKey = projectKey;
    }

    public String getProjectKey()
    {
        return this.projectKey;
    }

    public String getSummary()
    {
        return summary;
    }

    public void setSummary( final String summary )
    {
        this.summary = summary;
    }

    public String getDescription()
    {
        return description;
    }

    public void setDescription( final String description )
    {
        this.description = description;
    }

    public String getReporter()
    {
        return reporter;
    }

    public void setReporter( final String reporter )
    {
        this.reporter = reporter;
    }

    public String getAssignee()
    {
        return assignee;
    }

    public void setAssignee( final String assignee )
    {
        this.assignee = assignee;
    }

    public List<File> getProblemReportBundles()
    {
        return problemReportBundles;
    }

    public void addProblemReportBundle( final File problemReportBundle )
    {
        if ( problemReportBundle == null )
        {
            throw new IllegalArgumentException( "bundle file not specified" );
        }
        problemReportBundles.add( problemReportBundle );
    }

    @Deprecated
    public File getProblemReportBundle()
    {
        return problemReportBundles.isEmpty() ? null : problemReportBundles.get( 0 );
    }

    @Deprecated
    public void setProblemReportBundle( final File problemReportBundle )
    {
        this.problemReportBundles.clear();
        if ( problemReportBundle != null )
        {
            addProblemReportBundle( problemReportBundle );
        }
    }

    public List<File> getScreenCaptures()
    {
        return screenCaptures;
    }

    public void addScreenCapture( final File screenCapture )
    {
        if ( screenCapture == null )
        {
            throw new IllegalArgumentException( "screen capture file not specified" );
        }
        screenCaptures.add( screenCapture );
    }

    public void setEnvironment( final String environment )
    {
        this.environment = environment;
    }

    public String getEnvironment()
    {
        return environment;
    }

    public String getComponent()
    {
        return component;
    }

    public void setComponent( final String component )
    {
        this.component = component;
    }

    @Override
    public String toString()
    {
        StringBuffer sb = new StringBuffer();
        sb.append( "        Project: " ).append( projectKey ).append( "\n" );
        sb.append( "     Issue Type: " ).append( issueTypeId ).append( "\n" );
        sb.append( "        Summary: " ).append( summary ).append( "\n" );
        sb.append( "    Description: " ).append( description ).append( "\n" );
        sb.append( "      Component: " ).append( component ).append( "\n" );
        sb.append( "       Priority: " ).append( priorityId ).append( "\n" );
        sb.append( "Affects Version: " ).append( affectsVersionId ).append( "\n" );
        sb.append( "    Fix Version: " ).append( fixVersionId ).append( "\n" );
        sb.append( "       Assignee: " ).append( assignee ).append( "\n" );
        sb.append( "       Reporter: " ).append( reporter ).append( "\n" );
        sb.append( "    Environment: " ).append( environment ).append( "\n" );
        sb.append( "        Bundles: " ).append( problemReportBundles ).append( "\n" );
        sb.append( "       Captures: " ).append( screenCaptures ).append( "\n" );

        return sb.toString();
    }

    /**
     * Get all bundles currently associated with this request.
     */
    public Collection<Bundle> getBundles()
    {
        return bundles;
    }

    /**
     * Set the bundles currently associated with this request.
     */
    public void setBundles( final Collection<Bundle> bundles )
    {
        this.bundles = bundles;
    }

    public void setError( final Throwable error )
    {
        this.error = error;
    }

    public Throwable getError()
    {
        return error;
    }

    public boolean isAutofill()
    {
        return autofill;
    }

    public void setAutofill( final boolean autofill )
    {
        this.autofill = autofill;
    }

    public void setSkipSubmission( final boolean skipSubmission )
    {
        this.skipSubmission = skipSubmission;
    }

    public boolean isSkipSubmission()
    {
        return skipSubmission;
    }
    
    public Object getContext()
    {
        return context;
    }

    public void setContext( final Object context )
    {
        this.context = context;
    }
}
