/**
 *  Copyright (c) 2008-2011 Sonatype, Inc. All rights reserved.
 *
 *  This program is licensed to you under the Apache License Version 2.0,
 *  and you may not use this file except in compliance with the Apache License Version 2.0.
 *  You may obtain a copy of the Apache License Version 2.0 at http://www.apache.org/licenses/LICENSE-2.0.
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the Apache License Version 2.0 is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the Apache License Version 2.0 for the specific language governing permissions and limitations there under.
 */
package org.codehaus.plexus.swizzle;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.WeakHashMap;

import javax.inject.Inject;
import javax.inject.Named;
import javax.inject.Singleton;

import org.codehaus.plexus.swizzle.jira.authentication.AuthenticationSource;
import org.codehaus.plexus.util.StringUtils;
import org.codehaus.swizzle.jira.Issue;
import org.codehaus.swizzle.jira.Jira;
import org.codehaus.swizzle.jira.Project;
import org.codehaus.swizzle.jira.Version;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.sonatype.inject.Nullable;
import org.sonatype.jira.Attachment;
import org.sonatype.jira.AttachmentHandler;
import org.sonatype.jira.AttachmentResponse;
//import org.sonatype.jira.internal.JIRA412ScrapingAttachmentHandler;
//import org.sonatype.jira.internal.JIRA42ScrapingAttachmentHandler;
import org.sonatype.jira.internal.JIRARestAttachmentHandler;
import org.sonatype.sisu.issue.jira.JiraIssueRetriever;
import org.sonatype.sisu.pr.Modifier;
import org.sonatype.sisu.pr.ProjectManager;
import org.sonatype.sisu.pr.SystemEnvironmentContributor;
import org.sonatype.sisu.pr.bundle.Archiver;
import org.sonatype.sisu.pr.bundle.Bundle;
import org.sonatype.sisu.pr.bundle.BundleManager;
import org.sonatype.sisu.pr.bundle.Encryptor;
import org.sonatype.sisu.pr.bundle.internal.BundleAttachment;
import org.sonatype.sisu.pr.bundle.internal.FileAttachment;
import org.sonatype.sisu.pr.modifier.internal.TranslateComponentIdModifier;

@Singleton
@Named( "jira" )
public class JiraIssueSubmitter
    implements IssueSubmitter
{

    /** JIRA server URL to connect to. */
    private String serverUrl;

    private AuthenticationSource defaultCredentials;

    private final BundleManager bundleManager;

    private final ProjectManager projectManager;

    private final List<SystemEnvironmentContributor> environment;

    private final List<Modifier> modifiers;

    private final Archiver archiver;

    private final Logger logger = LoggerFactory.getLogger( JiraIssueSubmitter.class );

    private Encryptor encryptor;

    private String jiraVersion;

    private List<AttachmentHandler> attachmentHandlers;

    private Map<AuthenticationSource, Jira> jiraCache = new WeakHashMap<AuthenticationSource, Jira>(4);

    @Inject
    public JiraIssueSubmitter( final @Nullable ProjectManager projectManager,
                               final @Nullable BundleManager bundleManager,
                               final List<SystemEnvironmentContributor> environment, final List<Modifier> modifiers,
                               final Archiver archiver, final Encryptor encryptor,
                               final @Nullable List<AttachmentHandler> attachmentHandlers )
    {
        this.projectManager = projectManager;
        this.bundleManager = bundleManager;
        this.environment = environment;
        this.archiver = archiver;
        this.encryptor = encryptor;

        List<Modifier> sortedModifiers = new LinkedList<Modifier>( modifiers );
        Collections.sort( sortedModifiers, new Comparator<Modifier>()
        {

            @Override
            public int compare( final Modifier o1, final Modifier o2 )
            {
                return ( o1.getPriority() < o2.getPriority() ? 1 : ( o1.getPriority() == o2.getPriority() ? 0 : -1 ) );
            }

        } );

        this.modifiers = sortedModifiers;

        if ( attachmentHandlers == null )
        {
            this.attachmentHandlers =
                Arrays.<AttachmentHandler> asList( new JIRARestAttachmentHandler()
//                    new JIRA412ScrapingAttachmentHandler(), new JIRA42ScrapingAttachmentHandler()
                );
        }
        else
        {
            this.attachmentHandlers = attachmentHandlers;
        }
    }

    public JiraIssueSubmitter( final String serverUrl, final AuthenticationSource authenticationSource )
    {
        this.serverUrl = serverUrl;
        this.defaultCredentials = authenticationSource;
        this.projectManager = null;
        this.bundleManager = null;
        this.archiver = null;
        this.encryptor = null;
        this.modifiers = new LinkedList<Modifier>();

        JiraIssueRetriever retriever;

        try
        {
            // plexus-swizzle was doing that translation by default
            retriever = new JiraIssueRetriever( serverUrl, authenticationSource );

            modifiers.add( new Modifier()
            {
                // default plexus-swizzle 1.2 behavior

                @Override
                public IssueSubmissionRequest modify( final IssueSubmissionRequest request )
                {
                    if ( request.getIssueTypeId() == null )
                    {
                        request.setIssueTypeId( "2" );
                    }
                    if ( request.getPriorityId() == null )
                    {
                        request.setPriorityId( "1" );
                    }
                    return request;
                }

                @Override
                public int getPriority()
                {
                    return 0;
                }
            } );

            modifiers.add( new TranslateComponentIdModifier( retriever ) );
        }
        catch ( Exception e )
        {
            if ( logger.isDebugEnabled() )
            {
                logger.error( "Could not initialize JiraIssueRetriever: " + e.getMessage(), e );
            }
            else
            {
                logger.error( "Could not initialize JiraIssueRetriever: " + e.getMessage() );
            }
        }

        this.environment = Collections.emptyList();

        this.attachmentHandlers =
            Arrays.<AttachmentHandler> asList( new JIRARestAttachmentHandler()
//                                               new JIRA412ScrapingAttachmentHandler(),
//                new JIRA42ScrapingAttachmentHandler()
            );
    }

    @Inject
    public void setServerUrl( @Nullable @Named( "${pr.serverUrl}" ) String serverUrl )
    {
        if ( this.serverUrl == null || ( !this.serverUrl.equals( serverUrl ) ) )
        {
            this.serverUrl = serverUrl;
            jiraCache.clear();
        }
    }

    @Inject
    public void setCredentials( @Nullable AuthenticationSource credentials )
    {
        if ( this.defaultCredentials == null || ( !this.defaultCredentials.equals( credentials ) ) )
        {
            this.defaultCredentials = credentials;
            jiraCache.remove( credentials );
        }
    }

    @Override
    public String toString()
    {
        return String.format( "%s [%s]" , jiraVersion == null ? "Unknown Jira version" : jiraVersion, serverUrl);
    }

    /* UT */ Jira connect( final AuthenticationSource credentials )
        throws IssueSubmissionException
    {
        if ( this.jiraCache.containsKey( credentials ) )
        {
            return jiraCache.get( credentials );
        }

        try
        {
            Jira jira = new Jira( serverUrl );

            try
            {
                jira.login( credentials.getLogin(), credentials.getPassword() );
            }
            catch ( Exception e )
            {
                throw new IssueSubmissionException( String.format( "Login failed for server '%s' with username '%s'", serverUrl,
                                                                   credentials.getLogin() ), e );
            }

            this.jiraCache.put( credentials, jira );

            jiraVersion = "JIRA " + jira.getServerInfo().getVersion();

            return jira;
        }
        catch ( MalformedURLException e )
        {
            throw new IssueSubmissionException( "The URL '" + serverUrl + "' is not valid." );
        }
        catch ( Exception e )
        {
            if ( e instanceof IssueSubmissionException )
            {
                throw (IssueSubmissionException) e;
            }

            throw new IssueSubmissionException( String.format( "Connection failed (%s)",
                this ), e );
        }
    }

    @Override
    @Deprecated
    public IssueSubmissionResult submitIssue( final IssueSubmissionRequest request )
        throws IssueSubmissionException
    {
        return submit( request );
    }

    //
    // Issue Submission
    //
    @Override
    public IssueSubmissionResult submit( final IssueSubmissionRequest request )
        throws IssueSubmissionException
    {
        return submit( request, defaultCredentials );
    }

    @Override
    public IssueSubmissionResult submit( IssueSubmissionRequest request, final AuthenticationSource credentials )
        throws IssueSubmissionException
    {
        request.setReporter( credentials.getLogin() );
        request = modify( autofill( request ) );

        Jira jira = connect( credentials );

        if ( request.isValidateRequest() )
        {
            validateIssueSubmissionRequest( request, jira );
        }

        Project project = jira.getProject( request.getProjectKey() );

        Issue issue = new Issue();
        issue.setProject( project );
        issue.setSummary( request.getSummary() );
        issue.setDescription( request.getDescription() );
//        issue.setAssignee( jira.getUser( credentials.getLogin() ) );

        if ( request.getIssueTypeId() != null )
        {
            issue.setType( jira.getIssueType( Integer.parseInt( request.getIssueTypeId() ) ) );
        }

        if ( request.getAffectsVersionId() != null )
        {
            //
            // You cannot use the version object retrieved from the server, it yields a
            // "cannot create RemoteIssue from struct?" error.
            // So you have to make a new issues and just transfer the id. This is also faster because we don't have to
            // make a trip to
            // the server to get data. We've already done this in the client code to get the correct version id.
            //
            Version affectsVersion = new Version();
            affectsVersion.setId( Integer.parseInt( request.getAffectsVersionId() ) );
            issue.addAffectsVersion( affectsVersion );
        }

        //
        // Optional
        //

        if ( request.getPriorityId() != null )
        {
            issue.setPriority( jira.getPriority( Integer.parseInt( request.getPriorityId() ) ) );
        }

        if ( request.getEnvironment() != null )
        {
            issue.setEnvironment( request.getEnvironment() );
        }

        if ( request.getFixVersionId() != null )
        {
            //
            // Same as the affectsVersion
            //
            Version fixVersion = new Version();
            fixVersion.setId( Integer.parseInt( request.getFixVersionId() ) );
            issue.addFixVersion( fixVersion );
        }

        if ( request.getAssignee() != null )
        {
            issue.setAssignee( jira.getUser( request.getAssignee() ) );
        }

        if ( StringUtils.isNotEmpty( request.getComponent() ) )
        {
            org.codehaus.swizzle.jira.Component component = new org.codehaus.swizzle.jira.Component();
            int cId;
            try
            {
                cId = Integer.parseInt( request.getComponent() );
            }
            catch ( NumberFormatException e )
            {
                throw new IssueSubmissionException( "component name was not translated to ID: "
                    + request.getComponent(), e );
            }
            component.setId( cId );
            issue.addComponent( component );
        }

        //
        // We need to create an issue so that we can create an attachment. The XMLRPC API does not
        // allow for attachments so we have to use separate http client call to submit the attachment.
        //
        Issue addedIssue;

        try
        {
            addedIssue = jira.createIssue( issue );
        }
        catch ( Exception e )
        {
            throw new IssueSubmissionException( "Error creating issue: " + e.getMessage(), e );
        }

        AttachmentHandler handler = chooseAttachmentHandler( jiraVersion );
        processAttachments( addedIssue, request.getProblemReportBundles(), credentials, handler );
        processAttachments( addedIssue, request.getScreenCaptures(), credentials, handler );
        processAttachments( addedIssue, request.getBundles(), credentials, handler );

        return new IssueSubmissionResult( addedIssue.getLink(), addedIssue.getKey(), request.getBundles() );
    }

    private IssueSubmissionRequest autofill( final IssueSubmissionRequest request )
        throws IssueSubmissionException
    {
        if ( !request.isAutofill() )
        {
            return request;
        }

        if ( projectManager != null )
        {
            projectManager.fill( request );
        }

        if ( request.getBundles() == null && bundleManager != null )
        {
            request.setBundles( bundleManager.assemble( request ) );
        }
        if ( request.getEnvironment() == null )
        {
            String envString = "";
            for ( SystemEnvironmentContributor contributor : environment )
            {
                envString += contributor.asDiagnosticsFormat();
            }
            request.setEnvironment( envString.isEmpty() ? null : envString );
        }

        return request;
    }

    private IssueSubmissionRequest modify( IssueSubmissionRequest request )
    {
        for ( Modifier modifier : modifiers )
        {
            request = modifier.modify( request );
        }
        return request;
    }

    private void validateIssueSubmissionRequest( final IssueSubmissionRequest request, Jira jira )
        throws IssueSubmissionException
    {
        if ( !userExists( request.getReporter(), jira ) )
        {
            throw new IssueSubmissionException( "The reporter must exist in the JIRA users database. The user '"
                + request.getReporter() + "' does not exist." );
        }

        if ( StringUtils.isNotEmpty( request.getAssignee() ) && !userExists( request.getAssignee(), jira ) )
        {
            throw new IssueSubmissionException( "The assignee must exist in the JIRA users database. The user '"
                + request.getAssignee() + "' does not exist." );
        }
    }

    private boolean userExists( final String username, Jira jira1 )
    {
        if ( username != null && jira1.getUser( username ).getName() != null )
        {
            return true;
        }

        return false;
    }

    /* for UT */void processAttachments( final Issue addedIssue, final Collection<Bundle> bundles,
                                         final AuthenticationSource credentials )
        throws IssueSubmissionException
    {
        processAttachments( addedIssue, bundles, credentials, chooseAttachmentHandler( jiraVersion ) );
    }

    private void processAttachments( final Issue addedIssue, Collection<Bundle> bundles,
                                     final AuthenticationSource credentials, final AttachmentHandler handler )
        throws IssueSubmissionException
    {
        if ( bundles == null )
        {
            return;
        }

        if ( archiver != null )
        {
            Bundle archive;
            try
            {
                archive = archiver.createArchive( bundles );
                bundles = Collections.singleton( archive );
            }
            catch ( IOException e )
            {
                if ( logger.isDebugEnabled() )
                {
                    logger.warn( "Skipping creation of bundle archive: " + e.getMessage(), e );
                }
                else
                {
                    logger.warn( "Skipping creation of bundle archive: " + e.getMessage() );
                }
            }
        }

        for ( Bundle bundle : bundles )
        {
            try
            {
                AttachmentResponse response =
                    handler.attach( serverUrl, credentials.getLogin(), credentials.getPassword(),
                        String.valueOf( addedIssue.getId() ), addedIssue.getKey(),
                        Collections.<Attachment>singleton( new BundleAttachment( encrypt( bundle ) ) ) );
                if ( response.isErrorResponse() )
                {
                    throw new IssueSubmissionException( "Could not process attachments: " + response.getStatusText() );
                }
            }
            catch ( IOException e )
            {
                throw new IssueSubmissionException( "There was an error posting a problem report bundle: "
                    + bundle.getName(), e );
            }
        }
    }

    /**
     * Replace the bundle with an encrypted version. If encryption fails, just return original.
     */
    private Bundle encrypt( Bundle bundle )
    {
        if ( encryptor != null )
        {
            try
            {
                bundle = encryptor.encrypt( bundle );
            }
            catch ( IssueSubmissionException e )
            {
                if ( logger.isDebugEnabled() )
                {
                    logger.warn( "Skipping encryption of bundle: " + e.getMessage(), e );
                }
                else
                {
                    logger.warn( "Skipping encryption of bundle archive: {}", e.getMessage() );
                }
            }
        }
        return bundle;
    }

    /* for UT */void processAttachments( final Issue addedIssue, final List<File> attachments,
                                         final AuthenticationSource credentials )
        throws IssueSubmissionException
    {
        processAttachments( addedIssue, attachments, credentials, chooseAttachmentHandler( jiraVersion ) );
    }

    private void processAttachments( final Issue addedIssue, final List<File> attachments,
                                     final AuthenticationSource credentials, final AttachmentHandler handler )
        throws IssueSubmissionException
    {
        if ( attachments == null || attachments.isEmpty() )
        {
            return;
        }

        List<Attachment> items = new LinkedList<Attachment>();

        for ( File attachment : attachments )
        {
            Attachment att = new FileAttachment( attachment );
            items.add( att );
        }
        try
        {
            handler.attach( serverUrl, credentials.getLogin(), credentials.getPassword(),
                String.valueOf( addedIssue.getId() ), addedIssue.getKey(), items );
        }
        catch ( IOException e )
        {
            throw new IssueSubmissionException( "could not attach files: " + e.getMessage(), e );
        }
    }

    @Override
    public IssueSubmissionResult update( final Issue issue )
        throws IssueSubmissionException
    {
        return update( issue, defaultCredentials );
    }

    @Override
    public IssueSubmissionResult update( final Issue issue, final AuthenticationSource credentials )
        throws IssueSubmissionException
    {
        try
        {
            Jira jira = connect( credentials );
            Issue updated = jira.updateIssue( issue.getKey(), issue );
            return new IssueSubmissionResult( updated.getLink(), updated.getKey() );
        }
        catch ( Exception e )
        {
            throw new IssueSubmissionException( "Could not update issue " + issue.getKey(), e );
        }
    }

    private AttachmentHandler chooseAttachmentHandler( final String jiraVersion )
        throws IssueSubmissionException
    {
        for ( AttachmentHandler handler : attachmentHandlers )
        {
            if ( handler.isSupporting( jiraVersion ) )
            {
                return handler;
            }
        }

        throw new IssueSubmissionException( "Could not find attachment handler for JIRA version " + jiraVersion );
    }
}
