/**
 *  Copyright (c) 2008-2011 Sonatype, Inc. All rights reserved.
 *
 *  This program is licensed to you under the Apache License Version 2.0,
 *  and you may not use this file except in compliance with the Apache License Version 2.0.
 *  You may obtain a copy of the Apache License Version 2.0 at http://www.apache.org/licenses/LICENSE-2.0.
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the Apache License Version 2.0 is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the Apache License Version 2.0 for the specific language governing permissions and limitations there under.
 */
package org.sonatype.sisu.issue.jira;

import java.net.MalformedURLException;
import java.util.Arrays;
import java.util.List;

import javax.inject.Inject;
import javax.inject.Named;
import javax.inject.Singleton;

import org.codehaus.plexus.swizzle.jira.authentication.AuthenticationSource;
import org.codehaus.swizzle.jira.Component;
import org.codehaus.swizzle.jira.Filter;
import org.codehaus.swizzle.jira.Issue;
import org.codehaus.swizzle.jira.IssueType;
import org.codehaus.swizzle.jira.Jira;
import org.codehaus.swizzle.jira.Priority;
import org.codehaus.swizzle.jira.Project;
import org.codehaus.swizzle.jira.Version;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.sonatype.inject.Nullable;
import org.sonatype.sisu.issue.InitializationException;
import org.sonatype.sisu.issue.IssueRetrievalException;
import org.sonatype.sisu.issue.IssueRetriever;

/**
 * Retrieve issues using the injected server URL and credentials.
 * <p/>
 * 
 * <pre>
 * ${pr.serverUrl}: the server to connect to. Relies on an injected {@link AuthenticationSource}.
 * </pre>
 */
@Singleton
@Named( "jira" )
@SuppressWarnings( "unchecked" )
public class JiraIssueRetriever
    implements IssueRetriever
{
    // TODO add cache? swizzle is also caching, is that sufficient?

    /** Connection to the JIRA instance provided by Swizzle */
    private Jira jira;

    /** JIRA server URL to connect to. */
    private String serverUrl;

    private AuthenticationSource credentials;

    private final Logger logger;

    @Inject
    public JiraIssueRetriever( Logger logger )
    {
        this.logger = logger;
    }

    public JiraIssueRetriever( final String serverUrl, final AuthenticationSource authenticationSource )
        throws Exception
    {
        this.logger = LoggerFactory.getLogger( JiraIssueRetriever.class );
        this.serverUrl = serverUrl;
        this.credentials = authenticationSource;
    }

    @Override
    @Inject
    public void setServerUrl( @Named( "${pr.serverUrl}" ) @Nullable String serverUrl )
    {
        if ( this.serverUrl == null || ( !this.serverUrl.equals( serverUrl ) ) )
        {
            jira = null;
            this.serverUrl = serverUrl;
        }
    }

    @Override
    @Inject
    public void setCredentials( @Nullable AuthenticationSource credentials )
    {
        if ( this.credentials == null || ( !this.credentials.equals( credentials ) ) )
        {
            jira = null;
            this.credentials = credentials;
        }
    }

    //
    // Lifecycle
    //
    public void initialize()
        throws InitializationException
    {
        if ( jira != null )
        {
            return;
        }

        try
        {
            jira = new Jira( serverUrl );
            jira.login( credentials.getLogin(), credentials.getPassword() );
        }
        catch ( MalformedURLException e )
        {
            jira = null;
            throw new InitializationException( "The URL '" + serverUrl + "' is not valid." );
        }
        catch ( Exception e )
        {
            jira = null;
            throw new InitializationException( "The username and password combination is invalid.", e );
        }
    }

    private IssueRetrievalException wrap( final Exception e )
    {
        return new IssueRetrievalException( "Could not retrieve issues: " + e.getMessage(), e );
    }

    //
    // Issue Types
    //
    @Override
    public List<IssueType> getIssueTypes( final Project project )
    {
        initialize();
        return jira.getIssueTypesForProject( project.getKey() );
    }

    //
    // Components
    //
    @Override
    public List<org.codehaus.swizzle.jira.Component> getComponents( final Project project )
    {
        initialize();
        return jira.getComponents( project.getKey() );
    }

    //
    // Priorities
    //
    @Override
    public List<Priority> getPriorities()
    {
        initialize();
        return jira.getPriorities();
    }

    @Override
    public List<Issue> getIssues( final Filter filter )
        throws Exception
    {
        initialize();
        return getIssuesFromFilter( filter.getId() );
    }

    @Override
    public List<Version> getVersions( final Project project )
    {
        initialize();
        return jira.getVersions( project );
    }

    @Override
    public List<Filter> getFilters()
    {
        initialize();
        return jira.getFavoriteFilters();
    }

    @Override
    public List<Issue> getIssuesFromFilter( final int id )
        throws IssueRetrievalException
    {
        initialize();
        try
        {
            return jira.getIssuesFromFilter( id );
        }
        catch ( Exception e )
        {
            throw wrap( e );
        }
    }

    @Override
    public List<Issue> getIssues( final String search )
        throws IssueRetrievalException
    {
        initialize();
        try
        {
            return jira.getIssuesFromTextSearch( search );
        }
        catch ( Exception e )
        {
            throw wrap( e );
        }
    }

    @Override
    public List<Issue> getIssues( final String search, final Project project )
        throws IssueRetrievalException
    {
        initialize();
        return getIssues( search, project, 20 );
    }

    @Override
    public List<Issue> getIssues( final String search, final Project project, final int limit )
        throws IssueRetrievalException
    {
        initialize();
        try
        {
            return jira.getIssuesFromTextSearchWithProject( Arrays.asList( project.getKey() ), search, limit );
        }
        catch ( Exception e )
        {
            throw wrap( e );
        }
    }

    @Override
    public Project getProject( final String projectKey )
        throws IssueRetrievalException
    {
        initialize();
        Project project = jira.getProject( projectKey );
        if ( project == null )
        {
            throw new IssueRetrievalException( "Could not retrieve project for " + projectKey );
        }
        return project;
    }

    @Override
    public Priority getPriority( final int id )
    {
        initialize();
        return jira.getPriority( id );
    }

    @Override
    public Component getComponent( final Project project, final String component )
        throws IssueRetrievalException
    {
        initialize();
        Component c = jira.getComponent( project.getKey(), component );
        if ( c == null )
        {
            String msg = String.format( "Could not retrieve component '%s' for project '%s'", component, project );
            throw new IssueRetrievalException( msg );
        }
        return c;
    }
}
