/**
 *  Copyright (c) 2008-2011 Sonatype, Inc. All rights reserved.
 *
 *  This program is licensed to you under the Apache License Version 2.0,
 *  and you may not use this file except in compliance with the Apache License Version 2.0.
 *  You may obtain a copy of the Apache License Version 2.0 at http://www.apache.org/licenses/LICENSE-2.0.
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the Apache License Version 2.0 is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the Apache License Version 2.0 for the specific language governing permissions and limitations there under.
 */
package org.sonatype.sisu.pr;

import org.codehaus.plexus.swizzle.IssueSubmissionRequest;
import org.sonatype.sisu.pr.filler.internal.ThrowableIssueFillin;
import org.sonatype.sisu.pr.modifier.internal.SimilarIssuesModifier;
import org.sonatype.sisu.pr.modifier.internal.TranslateComponentIdModifier;

/**
 * A modifier is asked to inspect an {@link IssueSubmissionRequest} and may change values.
 *
 * @since 1.4
 */
public interface Modifier
{

    /**
     * Default priorities for modifiers.
     */
    public enum Priority
    {
        /**
         * Autofill of requests should happen first as this should not overwrite already set values.
         * <p/>
         * Example: Setting summary and description based on the error set in a request.
         * <p/>
         * Priority: 10
         *
         * @see ThrowableIssueFillin
         */
        FILLIN( 10 ),

        /**
         * After autofilling a request, the values may be modified.
         * <p/>
         * Example: Translating component name into numeric ID.
         * <p/>
         * Priority: 0
         *
         * @see TranslateComponentIdModifier
         */
        MODIFIER( 0 ),

        /**
         * Analyzing the request
         * <p/>
         * Example: Skipping request submission based on requests already submitted to the server.
         * <p/>
         * Priority: -10
         *
         * @see SimilarIssuesModifier
         */
        ANALYZER( -10 ), MINIMUM( Integer.MIN_VALUE );

        private final int priority;

        Priority( final int priority )
        {
            this.priority = priority;
        }

        /**
         * The actual numeric priority for comparison.
         */
        public int priority()
        {
            return priority;
        }

    };

    /**
     * Modify the given issue.
     *
     * @param request the request to inspect.
     * @return the modified request.
     */
    IssueSubmissionRequest modify( IssueSubmissionRequest request );

    /**
     * The priority of the modifier to determine the order in which all available modifiers will be executed.
     */
    int getPriority();
}
