/**
 *  Copyright (c) 2008-2011 Sonatype, Inc. All rights reserved.
 *
 *  This program is licensed to you under the Apache License Version 2.0,
 *  and you may not use this file except in compliance with the Apache License Version 2.0.
 *  You may obtain a copy of the Apache License Version 2.0 at http://www.apache.org/licenses/LICENSE-2.0.
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the Apache License Version 2.0 is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the Apache License Version 2.0 for the specific language governing permissions and limitations there under.
 */
package org.sonatype.sisu.pr.bundle;

import java.io.File;
import java.io.FileFilter;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.LinkedList;
import java.util.List;


public class FileBundle
    extends AbstractBundle
{

    private static final String DEFAULT_TYPE = "application/octet-stream";

    private final File file;

    private List<Bundle> bundles;

    private FileFilter filter;

    public FileBundle( final File file, final String type )
    {
        this( file, file.getName(), type );
    }

    public FileBundle( final File file, final String name, final String type )
    {
        super( name, type );
        this.file = file;
    }

    public FileBundle( final File file )
    {
        this( file, DEFAULT_TYPE );
    }

    @Override
    protected InputStream openStream()
        throws IOException
    {
        return new FileInputStream( file );
    }

    @Override
    public List<Bundle> getSubBundles()
    {
        if ( this.file.isDirectory() )
        {
            return generateSubBundles();
        }
        else
        {
            return null;
        }
    }

    private List<Bundle> generateSubBundles()
    {
        if ( bundles != null )
        {
            return bundles;
        }

        bundles = new LinkedList<Bundle>();
        File[] files = this.file.listFiles( filter );

        for ( File file : files )
        {
            this.bundles.add( new FileBundle( file ) );
        }

        return bundles;
    }

    public void setFilter( final FileFilter filter )
    {
        this.filter = filter;
    }

    @Override
    public long getContentLength()
    {
        return this.file.length();
    }

    @Override
    public String toString()
    {
        return getClass().getSimpleName() + " (" + this.getName() + ") [" + file.getPath() + "]";
    }

    @Override
    public int hashCode()
    {
        final int prime = 31;
        int result = 1;
        result = prime * result + ( ( bundles == null ) ? 0 : bundles.hashCode() );
        result = prime * result + ( ( file == null ) ? 0 : file.hashCode() );
        return result;
    }

    @Override
    public boolean equals( final Object obj )
    {
        if ( !super.equals( obj ) )
        {
            return false;
        }

        if ( this == obj )
        {
            return true;
        }
        if ( obj == null )
        {
            return false;
        }
        if ( getClass() != obj.getClass() )
        {
            return false;
        }
        FileBundle other = (FileBundle) obj;
        if ( bundles == null )
        {
            if ( other.bundles != null )
            {
                return false;
            }
        }
        else if ( !bundles.equals( other.bundles ) )
        {
            return false;
        }
        if ( file == null )
        {
            if ( other.file != null )
            {
                return false;
            }
        }
        else if ( !file.equals( other.file ) )
        {
            return false;
        }
        return true;
    }

}
