/**
 *  Copyright (c) 2008-2011 Sonatype, Inc. All rights reserved.
 *
 *  This program is licensed to you under the Apache License Version 2.0,
 *  and you may not use this file except in compliance with the Apache License Version 2.0.
 *  You may obtain a copy of the Apache License Version 2.0 at http://www.apache.org/licenses/LICENSE-2.0.
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the Apache License Version 2.0 is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the Apache License Version 2.0 for the specific language governing permissions and limitations there under.
 */
package org.sonatype.sisu.pr.bundle.internal;

import java.io.IOException;
import java.util.LinkedHashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.WeakHashMap;

import javax.inject.Inject;
import javax.inject.Named;
import javax.inject.Singleton;

import org.codehaus.plexus.swizzle.IssueSubmissionException;
import org.codehaus.plexus.swizzle.IssueSubmissionRequest;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.sonatype.sisu.pr.bundle.Archiver;
import org.sonatype.sisu.pr.bundle.Bundle;
import org.sonatype.sisu.pr.bundle.BundleAssembler;
import org.sonatype.sisu.pr.bundle.BundleManager;

@Named( "pr.default" )
@Singleton
public class DefaultBundleManager
    implements BundleManager
{

    private final List<BundleAssembler> assemblers;

    private final Map<IssueSubmissionRequest, Set<Bundle>> bundles =
        new WeakHashMap<IssueSubmissionRequest, Set<Bundle>>();

    private final Archiver archiver;

    private static final Logger logger = LoggerFactory.getLogger( DefaultBundleManager.class );

    @Inject
    public DefaultBundleManager( final List<BundleAssembler> assemblers, final Archiver archiver )
    {
        this.assemblers = assemblers;
        this.archiver = archiver;
    }

    @Override
    public Bundle createBundle( final IssueSubmissionRequest request )
        throws IOException, IssueSubmissionException
    {
        List<Bundle> bundles = assemble( request );
        Bundle archive = archiver.createArchive( bundles );
        return archive;
    }

    @Override
    public List<Bundle> assemble( final IssueSubmissionRequest request )
        throws IssueSubmissionException
    {
        return assemble( request, assemblers.toArray( new BundleAssembler[assemblers.size()] ) );
    }

    @Override
    public List<Bundle> assemble( final IssueSubmissionRequest request, final BundleAssembler... assemblers )
        throws IssueSubmissionException
    {
        for ( BundleAssembler assembler : assemblers )
        {
            if ( assembler.isParticipating( request ) )
            {
                Bundle bundle = assembler.assemble( request );
                logger.trace( "Assembled bundle: {}", bundle );
                addBundle( request, bundle );
            }
        }

        LinkedList<Bundle> ret = new LinkedList<Bundle>();
        Set<Bundle> bundles = this.bundles.get( request );
        if ( bundles != null )
        {
            ret.addAll( bundles );
        }

        logger.debug( "Assembled {} bundles for request {}", ret.size(), request );

        return ret;
    }

    @Override
    public void addBundle( final IssueSubmissionRequest request, final Bundle bundle )
    {
        Set<Bundle> bundles;
        if ( ( bundles = this.bundles.get( request ) ) == null )
        {
            this.bundles.put( request, ( bundles = new LinkedHashSet<Bundle>() ) );
        }
        bundles.add( bundle );
    }

}
