/**
 *  Copyright (c) 2008-2011 Sonatype, Inc. All rights reserved.
 *
 *  This program is licensed to you under the Apache License Version 2.0,
 *  and you may not use this file except in compliance with the Apache License Version 2.0.
 *  You may obtain a copy of the Apache License Version 2.0 at http://www.apache.org/licenses/LICENSE-2.0.
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the Apache License Version 2.0 is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the Apache License Version 2.0 for the specific language governing permissions and limitations there under.
 */
package org.sonatype.sisu.pr.bundle.internal;

import java.io.File;
import java.io.IOException;
import java.util.UUID;

import javax.inject.Inject;
import javax.inject.Named;
import javax.inject.Singleton;

import org.apache.commons.io.FileCleaningTracker;
import org.sonatype.inject.Nullable;
import org.sonatype.sisu.pr.bundle.Bundle;
import org.sonatype.sisu.pr.bundle.ManagedBundle;
import org.sonatype.sisu.pr.bundle.StorageManager;

/**
 * This storage manager deletes files when the bundle instance is garbage collected.
 *
 * @since 1.4
 */
@Named( "pr.default" )
@Singleton
public class ReapingFileStorageManager
    implements StorageManager
{

    private final File baseDir;

    private FileCleaningTracker tracker;

    @Inject
    public ReapingFileStorageManager( @Nullable @Named( "${pr.storage.baseDir}" ) final File baseDir )
        throws IOException
    {
        if ( baseDir != null && !"${pr.storage.baseDir}".equals( baseDir.getName() ) )
        {
            this.baseDir = baseDir;
        }
        else
        {
            this.baseDir = new File( System.getProperty( "java.io.tmpdir" ), getClass().getSimpleName() );
        }

        if ( !this.baseDir.exists() && !this.baseDir.mkdirs() )
        {
            throw new IOException( "Could not create base directory " + baseDir.getAbsolutePath().toString() );
        }
    }

    protected FileCleaningTracker tracker()
    {
        if ( tracker == null )
        {
            tracker = new FileCleaningTracker();
        }
        
        return tracker;
    }

    @Override
    public ManagedBundle createBundle( final String name, final String contentType )
    {
        File storageFile = new File( baseDir, UUID.randomUUID().toString().replace( ':', '_' ) );

        ManagedFileBundle bundle = new ManagedFileBundle( name, contentType, storageFile );
        tracker().track( storageFile, bundle );

        return bundle;
    }

    @Override
    public synchronized void release()
    {
        if ( this.tracker != null )
        {
            this.tracker.exitWhenFinished();
            this.tracker = null;
        }
    }

    @Override
    public void release( final Bundle bundle )
    {
    }

}
