/**
 *  Copyright (c) 2008-2011 Sonatype, Inc. All rights reserved.
 *
 *  This program is licensed to you under the Apache License Version 2.0,
 *  and you may not use this file except in compliance with the Apache License Version 2.0.
 *  You may obtain a copy of the Apache License Version 2.0 at http://www.apache.org/licenses/LICENSE-2.0.
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the Apache License Version 2.0 is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the Apache License Version 2.0 for the specific language governing permissions and limitations there under.
 */
package org.sonatype.sisu.pr.bundle.internal;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.io.OutputStream;
import java.security.PublicKey;
import java.util.Collection;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import javax.inject.Inject;
import javax.inject.Named;

import org.codehaus.plexus.swizzle.IssueSubmissionException;
import org.codehaus.plexus.util.IOUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.sonatype.inject.Nullable;
import org.sonatype.sisu.pr.bundle.Bundle;
import org.sonatype.sisu.pr.bundle.Encryptor;
import org.sonatype.sisu.pr.bundle.ManagedBundle;
import org.sonatype.sisu.pr.bundle.StorageManager;

/**
 * This encryptor uses plexus-encryptor to encrypt bundles.
 * <p/>
 *
 * <pre>
 * ${pr.encryptor.publicKeyPath}: Path to the public key to use. May be a resource or file path.
 * pr.encryptor.publicKey: Instance of {@link PublicKey}. If a key instance is set, it is preferred over the path-based lookup.
 * </pre>
 *
 * If none of the properties are set, a warning is logged and encryption is skipped but won't raise an error.
 */
@Named( "pr.default" )
public class RsaAesBundleEncryptor
    implements Encryptor
{

    private static final Logger logger = LoggerFactory.getLogger( RsaAesBundleEncryptor.class );

    private final PublicKey publicKey;

    private final String publicKeyPath;

    private final StorageManager storageManager;

    private final org.sonatype.sisu.encryptor.Encryptor encryptor;

    @Inject
    public RsaAesBundleEncryptor( final StorageManager storageManager,
                                  final org.sonatype.sisu.encryptor.Encryptor encryptor,
                                  final @Nullable @Named( "${pr.encryptor.publicKeyPath}" ) String publicKeyPath,
                                  final @Nullable @Named( "pr.encryptor.publicKey" ) PublicKey publicKey )
    {
        this.storageManager = storageManager;

        this.encryptor = encryptor;

        this.publicKey = publicKey;

        this.publicKeyPath = publicKeyPath;

        if ( publicKey == null && publicKeyPath == null )
        {
            logger.warn( "Neither PublicKey instance nor public key file available. Bundle encryption will be skipped." );
        }
    }

    @Override
    public Bundle encrypt( final Bundle bundle )
        throws IssueSubmissionException
    {
        Collection<Bundle> encrypted = encrypt( Collections.singleton( bundle ) );

        if ( encrypted == null || encrypted.isEmpty() )
        {
            return null;
        }

        return encrypted.iterator().next();
    }

    @Override
    public Collection<Bundle> encrypt( final Collection<Bundle> bundles )
        throws IssueSubmissionException
    {
        if ( publicKey == null && publicKeyPath == null )
        {
            logger.debug( "No public key available. Skipping bundle encryption." );
            return bundles;
        }

        List<Bundle> encryptedBundles = new LinkedList<Bundle>();
        for ( Bundle bundle : bundles )
        {

            OutputStream out = null;
            InputStream in = null;
            try
            {
                ManagedBundle encryptedBundle =
                    storageManager.createBundle( bundle.getName() + ".enc", "application/octet-stream" );
                out = encryptedBundle.getOutputStream();

                in = bundle.getInputStream();
                if ( publicKey != null )
                {
                    encryptor.encrypt( in, out, publicKey );
                }
                else if ( publicKeyPath != null )
                {
                    encryptor.encrypt( in, out, openPublicKeyStream() );
                }

                encryptedBundles.add( encryptedBundle );
            }
            catch ( Exception e )
            {
                String msg =
                    String.format( "Cannot encrypt bundle %s (%s): ", bundle.getName(), bundle.getContentType() );
                throw new IssueSubmissionException( msg + e.getMessage() );
            }
            finally {
                IOUtil.close( out );
                IOUtil.close( in );
            }

        }

        return encryptedBundles;
    }

    private InputStream openPublicKeyStream()
        throws FileNotFoundException
    {
        InputStream stream = getClass().getResourceAsStream( publicKeyPath );
        if ( stream == null )
        {
            return new FileInputStream( publicKeyPath );
        }
        return stream;
    }

}
