/**
 *  Copyright (c) 2008-2011 Sonatype, Inc. All rights reserved.
 *
 *  This program is licensed to you under the Apache License Version 2.0,
 *  and you may not use this file except in compliance with the Apache License Version 2.0.
 *  You may obtain a copy of the Apache License Version 2.0 at http://www.apache.org/licenses/LICENSE-2.0.
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the Apache License Version 2.0 is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the Apache License Version 2.0 for the specific language governing permissions and limitations there under.
 */
package org.sonatype.sisu.pr.bundle.internal;

import java.io.File;
import java.io.IOException;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import javax.annotation.Nullable;
import javax.inject.Inject;
import javax.inject.Named;

import org.sonatype.sisu.pr.bundle.Bundle;
import org.sonatype.sisu.pr.bundle.ManagedBundle;
import org.sonatype.sisu.pr.bundle.StorageManager;

/**
 * A storage manager implementation without automatic cleanup of managed files.
 *
 * @see #release()
 *
 * @since 1.5
 */
public class TmpFileStorageManager
    implements StorageManager
{

    private final Set<File> files = new HashSet<File>();
    
    private final File tmpDir;

    public TmpFileStorageManager()
    {
        tmpDir = new File( System.getProperty( "java.io.tmpdir" ) );
    }

    @Inject
    public TmpFileStorageManager( @Nullable @Named("${pr.storage.baseDir}") final File tmpDir )
    {
        String path;
        if ( tmpDir == null )
        {
            this.tmpDir = new File( System.getProperty( "java.io.tmpdir", this.getClass().getSimpleName() ) );
        }
        else
        {
            this.tmpDir = tmpDir;
        }
    }

    @Override
    public ManagedBundle createBundle( final String name, final String contentType )
        throws IOException
    {
        createTmpDirIfNecessary();
        
        File file = new File( tmpDir,  getClass().getSimpleName() + ".tmp" );
        files.add( file );
        return new ManagedFileBundle( name, contentType, file );
    }

    private void createTmpDirIfNecessary()
        throws IOException
    {
        if ( !tmpDir.canRead() && !tmpDir.mkdirs() )
        {
            throw new IOException( "Cannot create temporary directory: " + tmpDir );
        }
    }

    @Override
    public void release()
    {
        final Iterator<File> iterator = files.iterator();
        while ( iterator.hasNext() )
        {
            File file = iterator.next();
            if ( !file.exists() || file.delete() )
            {
                iterator.remove();
            }
        }
        if ( !files.isEmpty() )
        {
            final String message = "Could not delete files: " + files.toString();

            files.clear();

            throw new RuntimeException( message );
        }

        tmpDir.delete();
    }

    /**
     * Unsupported, use #release()
     */
    @Override
    public void release( final Bundle bundle )
    {
        throw new UnsupportedOperationException();
    }

}
