/**
 *  Copyright (c) 2008-2011 Sonatype, Inc. All rights reserved.
 *
 *  This program is licensed to you under the Apache License Version 2.0,
 *  and you may not use this file except in compliance with the Apache License Version 2.0.
 *  You may obtain a copy of the Apache License Version 2.0 at http://www.apache.org/licenses/LICENSE-2.0.
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the Apache License Version 2.0 is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the Apache License Version 2.0 for the specific language governing permissions and limitations there under.
 */
package org.sonatype.sisu.pr.bundle.internal;

import java.io.IOException;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import javax.inject.Inject;
import javax.inject.Named;
import javax.inject.Singleton;

import org.codehaus.plexus.util.IOUtil;
import org.sonatype.sisu.pr.bundle.Archiver;
import org.sonatype.sisu.pr.bundle.Bundle;
import org.sonatype.sisu.pr.bundle.ManagedBundle;
import org.sonatype.sisu.pr.bundle.StorageManager;

/**
 * A bundle archiver creating a compressed ZIP file.
 * <p/>
 * No entry will be added for a bundle containing sub bundles, it will only be used as a directory name.
 */
@Named
@Singleton
public class ZipArchiver
    implements Archiver
{

    private static final String CONTENT_TYPE = "application/zip";

    private final StorageManager storageManager;

    @Inject
    public ZipArchiver( final StorageManager manager )
    {
        this.storageManager = manager;
    }

    @Override
    public Bundle createArchive( final Bundle bundle )
        throws IOException
    {
        return createArchive( Collections.singleton( bundle ) );
    }

    @Override
    public Bundle createArchive( final Collection<Bundle> bundles )
        throws IOException
    {
        ZipOutputStream zip = null;
        try
        {
            ManagedBundle zippedBundle = storageManager.createBundle( "ProblemReportBundle.zip", getContentType() );


            zip = new ZipOutputStream( zippedBundle.getOutputStream() );

            for ( Bundle bundle : bundles )
            {
                add( "", bundle, zip );
            }

            zip.close();

            return zippedBundle;
        }
        finally
        {
            IOUtil.close( zip );
        }
    }

    private void add( final String prefix, final Bundle bundle, final ZipOutputStream zip )
        throws IOException
    {
        String dir = "";
        if ( prefix != null && prefix.length() > 0 )
        {
            dir = prefix + "/";
        }

        List<Bundle> subBundles = bundle.getSubBundles();
        if ( subBundles != null )
        {
            for ( Bundle subBundle : subBundles )
            {
                add( dir + bundle.getName(), subBundle, zip );
            }
        }
        else
        {
            zip.putNextEntry( new ZipEntry( dir + bundle.getName() ) );
            IOUtil.copy( bundle.getInputStream(), zip );
        }
    }

    @Override
    public String getContentType()
    {
        return CONTENT_TYPE;
    }

}
