/**
 *  Copyright (c) 2008-2011 Sonatype, Inc. All rights reserved.
 *
 *  This program is licensed to you under the Apache License Version 2.0,
 *  and you may not use this file except in compliance with the Apache License Version 2.0.
 *  You may obtain a copy of the Apache License Version 2.0 at http://www.apache.org/licenses/LICENSE-2.0.
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the Apache License Version 2.0 is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the Apache License Version 2.0 for the specific language governing permissions and limitations there under.
 */
package org.sonatype.sisu.pr.internal;

import java.util.List;

import javax.inject.Inject;
import javax.inject.Named;
import javax.inject.Singleton;

import org.codehaus.plexus.swizzle.IssueSubmissionException;
import org.codehaus.plexus.swizzle.IssueSubmissionRequest;
import org.codehaus.plexus.swizzle.jira.authentication.AuthenticationSource;
import org.codehaus.swizzle.jira.Component;
import org.codehaus.swizzle.jira.Project;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.sonatype.inject.Nullable;
import org.sonatype.sisu.issue.IssueRetrievalException;
import org.sonatype.sisu.issue.IssueRetriever;
import org.sonatype.sisu.pr.ProjectManager;

/**
 * ProjectManager using properties to set project and component.
 * <p/>
 * 
 * <pre>
 * ${pr.project}: the project key (e.g. SBOX)
 * ${pr.component}: the component id. If this is not a number, this implementation will try to lookup the component and get it's id.
 * ${pr.serverUrl}: the server to use for lookups. This also relies on an injected {@link AuthenticationSource} instance.
 * </pre>
 * 
 * @since 1.4
 */
@Named( "pr.default" )
@Singleton
public class DefaultProjectManager
    implements ProjectManager
{
    private static final Logger logger = LoggerFactory.getLogger( DefaultProjectManager.class );

    private final String project;

    private final String component;

    private int componentId;

    private IssueRetriever issueRetriever;

    public static final String NO_COMPONENT = "";

    public static final String NO_PROJECT = "";

    @Inject
    public DefaultProjectManager( final @Nullable @Named( "${pr.project}" ) String project,
                                  final @Nullable @Named( "${pr.component}" ) String component,
                                  final @Nullable IssueRetriever issueRetriever )
        throws IssueSubmissionException
    {
        this.issueRetriever = issueRetriever;

        if ( "${pr.project}".equals( project ) )
        {
            this.project = NO_PROJECT;
        }
        else
        {
            this.project = project;
        }

        if ( component == null && !"${pr.component}".equals( component ) )
        {
            this.component = NO_COMPONENT;
        }
        else
        {
            this.component = component;
            try
            {
                int id = Integer.parseInt( component );
                this.componentId = id;
            }
            catch ( NumberFormatException e )
            {
                if ( issueRetriever == null )
                {
                    String msg =
                        String.format( "IssueRetriever missing: Cannot lookup component id for '%s'.", component );
                    throw new IssueSubmissionException( msg );
                }
                this.componentId = -1;
            }
        }
    }

    private void lookupComponentId()
        throws IssueSubmissionException
    {
        logger.debug( "trying to find component id for '{}'", component );

        Project p;
        try
        {
            p = issueRetriever.getProject( project );
            if ( p != null )
            {
                List<Component> components = issueRetriever.getComponents( p );
                for ( Component c : components )
                {
                    if ( component.equals( c.getName() ) )
                    {
                        this.componentId = c.getId();
                        return;
                    }
                }
            }
        }
        catch ( IssueRetrievalException e )
        {
            logger.debug( e.getMessage(), e );
            logger.warn( "Could not find component id for component {}", component );
        }
        throw new IssueSubmissionException( String.format( "Could not find component id for component '%s'", component ) );
    }

    @Override
    public String getProject( final IssueSubmissionRequest request )
    {
        return project;
    }

    @Override
    public int getComponentId( final IssueSubmissionRequest request )
        throws IssueSubmissionException
    {
        if ( componentId == -1 )
        {
            if ( issueRetriever != null )
            {
                lookupComponentId();
            }
        }
        return componentId;
    }

    @Override
    public IssueSubmissionRequest fill( final IssueSubmissionRequest request )
        throws IssueSubmissionException
    {
        if ( request.getProjectKey() == null )
        {
            request.setProjectKey( getProject( request ) );
        }
        if ( request.getComponent() == null )
        {
            request.setComponent( String.valueOf( getComponentId( request ) ) );
        }
        return request;
    }
}
