/**
 *  Copyright (c) 2008-2011 Sonatype, Inc. All rights reserved.
 *
 *  This program is licensed to you under the Apache License Version 2.0,
 *  and you may not use this file except in compliance with the Apache License Version 2.0.
 *  You may obtain a copy of the Apache License Version 2.0 at http://www.apache.org/licenses/LICENSE-2.0.
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the Apache License Version 2.0 is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the Apache License Version 2.0 for the specific language governing permissions and limitations there under.
 */
package org.sonatype.sisu.pr.internal;

import java.util.Formatter;

import javax.inject.Inject;
import javax.inject.Named;
import javax.inject.Singleton;

import org.sonatype.sisu.pr.SystemEnvironmentContributor;

/**
 * Formatters must be null safe and not throw Exceptions in that case.
 *
 * @since 1.4
 */
@Named( "pr.default" )
@Singleton
public class DefaultSystemEnvironmentContributor implements SystemEnvironmentContributor
{
    private final SystemEnvironment environment;

    /**
     * Used to repeatably call asXxxFormat while the underlying environment changes.
     */
    @Inject
    public DefaultSystemEnvironmentContributor( final SystemEnvironment environment )
    {
        this.environment = environment;
    }

    @Override
    public String asDiagnosticsFormat()
    {
        String keyValueFormat = "%s: %s%n";

        Formatter formatter = new Formatter();

        formatter.format( keyValueFormat, "java.vendor", environment.getJavaVendor() );
        formatter.format( keyValueFormat, "java.version", environment.getJavaVersion() );

        formatter.format( keyValueFormat, "os.name", environment.getOsName() );
        formatter.format( keyValueFormat, "os.version", environment.getOsVersion() );
        formatter.format( keyValueFormat, "os.arch", environment.getOsArch() );

        formatter.format( keyValueFormat, "runtime.available-processors", environment.getAvailableProcessors() );

        ByteFormatter bf = new ByteFormatter( ByteFormatter.Unit.SI );
        formatter.format( keyValueFormat, "runtime.max-memory", bf.humanReadableByteCount( environment.getMaxMemory() ) );
        formatter.format( keyValueFormat, "runtime.total-memory",
                          bf.humanReadableByteCount( environment.getTotalMemory() ) );
        formatter.format( keyValueFormat, "runtime.free-memory",
                          bf.humanReadableByteCount( environment.getFreeMemory() ) );

        return formatter.toString();
    }

    public static String asDiagnosticsFormat( final SystemEnvironment environment )
    {
        return new DefaultSystemEnvironmentContributor( environment ).asDiagnosticsFormat();
    }

    /**
     * @since 1.4
     */
    private static class ByteFormatter
    {
        private final Unit unit;

        ByteFormatter( final Unit unit )
        {
            this.unit = unit;
        }

        enum Unit
        {
            SI( true ), BINARY( false );

            private final boolean isSi;

            private Unit( final boolean isSi )
            {
                this.isSi = isSi;
            }

            public boolean isSi()
            {
                return isSi;
            }
        }

        /**
         * Based on implementation by aioobe [1] from question by unwind [2] on Stack Overflow [3]. [1]
         * http://stackoverflow.com/users/276052/aioobe [2] http://stackoverflow.com/users/28169/unwind] [3]
         * http://stackoverflow.com/questions/3758606/how-to-convert-byte-size-into-human-readable-format-in-java
         */
        public String humanReadableByteCount( final long bytes )
        {
            boolean si = unit.isSi();
            int byteCount = si ? 1000 : 1024;
            if ( bytes < byteCount )
            {
                return bytes + " B";
            }
            int exp = (int) ( Math.log( bytes ) / Math.log( byteCount ) );
            String pre = ( si ? "kMGTPE" : "KMGTPE" ).charAt( exp - 1 ) + ( si ? "" : "i" );
            return String.format( "%.1f %sB", bytes / Math.pow( byteCount, exp ), pre );
        }
    }
}
