/**
 *  Copyright (c) 2008-2011 Sonatype, Inc. All rights reserved.
 *
 *  This program is licensed to you under the Apache License Version 2.0,
 *  and you may not use this file except in compliance with the Apache License Version 2.0.
 *  You may obtain a copy of the Apache License Version 2.0 at http://www.apache.org/licenses/LICENSE-2.0.
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the Apache License Version 2.0 is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the Apache License Version 2.0 for the specific language governing permissions and limitations there under.
 */
package org.sonatype.sisu.pr.modifier.internal;

import java.util.List;

import javax.inject.Named;

import org.codehaus.plexus.swizzle.IssueSubmissionException;
import org.codehaus.plexus.swizzle.IssueSubmissionRequest;
import org.codehaus.plexus.swizzle.IssueSubmitter;
import org.codehaus.swizzle.jira.Issue;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.sonatype.inject.Nullable;
import org.sonatype.sisu.issue.IssueRetrievalException;
import org.sonatype.sisu.issue.IssueRetriever;
import org.sonatype.sisu.pr.Modifier;

//@Named
public class SimilarIssuesModifier
    implements Modifier
{

    private static final Logger logger = LoggerFactory.getLogger( SimilarIssuesModifier.class );

    private final IssueRetriever retriever;

    private final Integer priorityId;

    private final IssueSubmitter submitter;

//    @Inject
    public SimilarIssuesModifier( final IssueRetriever retriever,
                                  final IssueSubmitter issueSubmitter,
                                  @Nullable @Named( "${pr.modifier.similarissue.newPriorityId}" ) final String priorityId )
    {
        this.retriever = retriever;
        if ( priorityId != null && !"${pr.modifier.similarissue.newPriorityId}".equals( priorityId ) )
        {
            this.priorityId = Integer.valueOf( priorityId );
        }
        else
        {
            this.priorityId = null;
        }
        this.submitter = issueSubmitter;
    }

    @Override
    public IssueSubmissionRequest modify( final IssueSubmissionRequest request )
    {
        try
        {
            List<Issue> issues;
            if ( request.getProjectKey() != null )
            {
                issues = retriever.getIssues( request.getSummary(), retriever.getProject( request.getProjectKey() ) );
            }
            else
            {
                issues = retriever.getIssues( request.getSummary() );
            }

            if ( !issues.isEmpty() )
            {
                request.setSkipSubmission(true);
            }

            if ( issues.size() >= 20 )
            {
                issues = issues.subList( 0, 20 );
            }

            for ( Issue issue : issues )
            {
                if ( priorityId != null )
                {
                    updatePriority( issue, priorityId );
                }
                else
                {
                    raisePriority( issue );
                }
            }
        }
        catch ( IssueRetrievalException e )
        {
            if ( logger.isDebugEnabled() )
            {
                logger.warn( "Could not execute online search to check for similar issues", e );
            }
            else
            {
                logger.warn( "Could not execute online search to check for similar issues" );
            }
        }

        return request;
    }

    private void raisePriority( final Issue issue )
    {
        int id = issue.getPriority().getId();
        int lastId = Integer.MIN_VALUE;

        // the assumption of a descending severity-sorted list holds at least for the sonatype JIRA
        List<org.codehaus.swizzle.jira.Priority> priorities = retriever.getPriorities();
        for ( org.codehaus.swizzle.jira.Priority priority : priorities )
        {
            int serverid = priority.getId();
            if ( serverid == id )
            {
                break;
            }

            lastId = serverid;
        }

        if ( lastId == Integer.MIN_VALUE )
        {
            return;
        }

        updatePriority( issue, lastId );
    }

    private void updatePriority( final Issue issue, final Integer id )
    {
        issue.setPriority( retriever.getPriority( id.intValue() ) );

        try
        {
            submitter.update( issue );
        }
        catch ( IssueSubmissionException e )
        {
            if ( logger.isDebugEnabled() )
            {
                logger.warn( String.format( "Could not modify priority of existing similar issue: %s, update to priority id %s",
                                            issue, id ), e );
            }
            else
            {
                logger.warn( "Could not modify priority of existing similar issue: {}, update to priority id {}",
                             issue, id );
            }
        }
    }

    @Override
    public int getPriority()
    {
        return Priority.ANALYZER.priority();
    }

}
